function [u,p,on,sn,n] = l2_DTGV(b,A,AT,N,lambda,th,a,Ni,tol)
% Solve the l2-DTGV problem:
% min_u J(u) := 1/2*|Au - b|_2^2 + DTGV_lambda(u),
% using the Primal-Dual-Hybrid-Gradient method by Chambolle and Pock.
%
% Input
% b:        data (e.g. noisy) stacked in column vector
% A:        system matrix/forward operator size N^2 x length(b)
% AT:       transpose of system matrix
% N:        width and height of 2D function u (assumed to be square)
% lambda:   regularization parameters, [lambda1,lambda2]
% th:       angle for DTV regularizer in degrees
% a:        width parameter for DTV, should be in ]0,1]
% Ni:       maximum number of iterations
% tol:      tollerance for stopping criterion (relative change of J(u)
%
% Output
% u:        2D function solution, size N x N
% on:       relative change of J(u) for each iteration
% un:       relative change of u for each iteration
% n:        number of used iterations
%
% By Rasmus Dalgas Kongskov, 10/05/2016, DTU Compute

% finite diffrence derivative approximation matrix
Df = sparse(-eye(N)+diag(ones(N-1,1),1)); Df(end,end)=0; % forward diff.
Db = sparse(eye(N)-diag(ones(N-1,1),-1)); Db(1,1)=0; % backward diff.
Dx = kron(Df,eye(N));                           % derivative in x-direction
if th>=0
    Dy = kron(eye(N),Df);                       % derivative in y-direction
else
    Dy = kron(eye(N),Db);                       % derivative in y-direction
end

% power method to estimate |K|
ux = ones(N^2,1);
for n = 1:50
    ux = AT*(A*ux);
    ux = ux/norm(ux);
end
nA = norm(A*ux);

% scale to operator-norm 1
K  = @(ui) double(1/nA*A*ui);
KT = @(qi) double(1/nA*AT*qi);
b  = 1/nA*b;

% parameters from Chambolle-Pock
L       = 1/2*(sqrt(32) + 18);
tau     = 1 * 1/sqrt(L*1.01);
sigma   = 1 * 1/sqrt(L*1.01);
epsilon = 0;                                % smoothing parameter

% weights
lambda0 = lambda(1);
lambda1 = lambda(2);

% initialization
u = zeros(N^2,1); ub = u;
p = zeros(N^2,2); pb = p; v  = p;
w = zeros(N^2,3);
q = b*0;

on = zeros(Ni,1); sn = on;

% Ellipse gradient and divergence projection operators
th = th/180*pi;
c   = cos(th);
s   = sin(th);

Exf  = @(uin) c*Dx*uin - s*Dy*uin;
Eyf  = @(uin) a*(c*Dy*uin + s*Dx*uin);

Exb  = @(pin) -Dx'*(c*pin) + Dy'*(s*pin);
Eyb  = @(pin) a*(-Dy'*(c*pin) - Dx'*(s*pin));

for n = 1:Ni
    
    fa = 1+sigma*epsilon;
    
    % step in v
    v(:,1) = v(:,1) + sigma*(Exf(ub) - pb(:,1) );
    v(:,2) = v(:,2) + sigma*(Eyf(ub) - pb(:,2) );
    ma1 = max(1,sqrt(v(:,1).^2 + v(:,2).^2)/(lambda1*fa));
    v(:,1) = (v(:,1)/fa)./ma1;
    v(:,2) = (v(:,2)/fa)./ma1;
    
    % step in w
    w(:,1) = w(:,1) + sigma*(Exb(pb(:,1)));                   % w11
    w(:,2) = w(:,2) + sigma*1/2*(Eyb(pb(:,1))+Exb(pb(:,2)));  % w12
    w(:,3) = w(:,3) + sigma*(Eyb(pb(:,2)));                   % w22
    ma0 = max(1,sqrt(w(:,1).^2 + 2*w(:,2).^2 + w(:,3).^2)/(lambda0*fa));
    w(:,1) = (w(:,1)/fa)./ma0;
    w(:,2) = (w(:,2)/fa)./ma0;
    w(:,3) = (w(:,3)/fa)./ma0;
    
    % step in q
    q = (q + sigma*(K(ub)-b))/(1+sigma);
    
    % save current u and p values
    ub = u;
    pb = p;
    
    % step in u
    u = u + tau*(Exb(v(:,1)) + Eyb(v(:,2)) - tau*KT(q));
    
    % step in p
    p(:,1) = p(:,1) + tau*(v(:,1) + Exf(w(:,1)) + Eyf(w(:,2)));
    p(:,2) = p(:,2) + tau*(v(:,2) + Exf(w(:,2)) + Eyf(w(:,3)));
    
    % objective-function- and solution-change
    on(n) = nA/2*norm(K(u)-b)^2 ...
        + lambda1*sum(sqrt( (Exf(u)-p(:,1)).^2 + (Eyf(u)-p(:,2)).^2)) ...
        + lambda0*sum(sqrt( Exb(p(:,1)).^2 ...
        + 1/2*( Eyb(p(:,1)) + Exb(p(:,2)) ).^2 + Eyb(pb(:,2)).^2));
    
    sn(n)  = norm(u-ub)/norm(ub);
    
    % stop if relative objective function change is below tol
    if n>1 && (abs(on(n)-on(n-1))/on(n-1))<tol;
        break
    end
    
    % step in ub and pb
    ub = 2*u - ub;
    pb = 2*p - pb;
    
end

u  = reshape(u,N,N);
pt = zeros(N,N,2);
pt(:,:,1) = reshape(p(:,1),N,N);
pt(:,:,2) = reshape(p(:,2),N,N);
p = pt;

on = on(1:n); sn = sn(1:n);
on = abs(diff([2*on(1);on]))./on;

end